/* _NVRM_COPYRIGHT_BEGIN_
 *
 * Copyright 2013-2014 by NVIDIA Corporation.  All rights reserved.  All
 * information contained herein is proprietary and confidential to NVIDIA
 * Corporation.  Any use, reproduction, or disclosure without the written
 * permission of NVIDIA Corporation is prohibited.
 *
 * _NVRM_COPYRIGHT_END_
 */


/*
 * nv_gpu_ops.h
 *
 * This file defines the interface between the common RM layer
 * and the OS specific platform layers. (Currently supported 
 * are Linux and KMD)
 *
 */

#ifndef _NV_GPU_OPS_H_
#define _NV_GPU_OPS_H_
#include "nvgputypes.h"

typedef struct gpuSession      *gpuSessionHandle;
typedef struct gpuAddressSpace *gpuAddressSpaceHandle;
typedef struct gpuChannel      *gpuChannelHandle;
typedef struct gpuObject       *gpuObjectHandle;

struct gpuChannelInfo 
{
   volatile unsigned * GPGet;
   volatile unsigned * GPPut;
   NvU64*   gpFifoEntries;
   unsigned numGpFifoEntries;   
   unsigned channelClassNum;
   NvNotification * errorNotifier;
   NvBool *eccErrorNotifier;
} ;

struct gpuCaps
{
    unsigned largePageSize;
    unsigned smallPageSize;
    unsigned eccMask;
    unsigned eccOffset;
    void   * eccReadLocation;
    unsigned pcieSpeed;
    unsigned pcieWidth;
    NvBool   bEccEnabled;
};

struct gpuAllocInfo
{
    NvU64       rangeBegin;
    NvU64       rangeEnd;
    NvU64       gpuPhysOffset;
    NvBool      bContiguousPhysAlloc;
    NvBool      bHandleProvided;
    NvHandle    hPhysHandle;
};

NV_STATUS nvGpuOpsCreateSession(gpuSessionHandle *session);

void nvGpuOpsDestroySession(gpuSessionHandle session);

NV_STATUS nvGpuOpsAddressSpaceCreate(gpuSessionHandle session,
                                     unsigned long long uuidMsb, 
                                     unsigned long long uuidLsb, 
                                     gpuAddressSpaceHandle *vaSpace,
                                     unsigned long long vaBase,
                                     unsigned long long vaSize);

NV_STATUS nvGpuOpsAddressSpaceCreateMirrored(gpuSessionHandle session,
                                     unsigned long long uuidMsb, 
                                     unsigned long long uuidLsb, 
                                     gpuAddressSpaceHandle *vaSpace);

NV_STATUS nvGpuOpsAddressSpaceDestroy(gpuAddressSpaceHandle vaSpace);

NV_STATUS nvGpuOpsMemoryAllocFb (gpuAddressSpaceHandle vaSpace,
    NvLength length, NvU64 *gpuOffset, struct  gpuAllocInfo * allocInfo);

NV_STATUS nvGpuOpsMemoryAllocSys (gpuAddressSpaceHandle vaSpace,
    NvLength length, NvU64 *gpuOffset, struct gpuAllocInfo * allocInfo);

NV_STATUS nvGpuOpsChannelAllocate(gpuAddressSpaceHandle vaSpace,
    gpuChannelHandle  *channelHandle, struct gpuChannelInfo *channelInfo);

void nvGpuOpsChannelDestroy(struct gpuChannel *channel);

void nvGpuOpsMemoryFree(gpuAddressSpaceHandle vaSpace,
     NvU64 pointer);

NV_STATUS  nvGpuOpsMemoryCpuMap(gpuAddressSpaceHandle vaSpace,
                      NvU64 memory, NvLength length,  void **cpuPtr);

void nvGpuOpsMemoryCpuUnMap(gpuAddressSpaceHandle vaSpace,
     void* cpuPtr);

NV_STATUS nvGpuOpsCopyEngineAllocate(gpuChannelHandle channel,
          unsigned ceIndex, unsigned *class, gpuObjectHandle *copyEngineHandle);

NV_STATUS nvGpuOpsQueryCaps(struct gpuAddressSpace *vaSpace,
                            struct gpuCaps *caps);

const char  *nvGpuOpsChannelTranslateError(unsigned info32);


NV_STATUS nvGpuOpsGetHandles(NvHandle hClient, NvHandle hDevice, NvHandle hSubDevice,
                                    struct gpuSession **session, 
                                    struct gpuAddressSpace **vaSpace);

void nvGpuOpsDestroyHandles(struct gpuSession *session,
                            struct gpuAddressSpace *vaSpace);

NV_STATUS nvGpuOpsDupAllocation(NvHandle hPhysHandle, 
                                struct gpuAddressSpace *sourceVaspace, 
                                NvU64 sourceAddress,
                                struct gpuAddressSpace *destVaspace,
                                NvU64 *destAddress);
                                
NV_STATUS nvGpuOpsGetGuid(NvHandle hClient, NvHandle hDevice, 
                          NvHandle hSubDevice, NvU8 *gpuGuid, 
                          unsigned guidLength);
                          
NV_STATUS nvGpuOpsGetClientInfoFromPid(unsigned pid, NvU8 *gpuUuid, 
                                       NvHandle *hClient, 
                                       NvHandle *hDevice,
                                       NvHandle *hSubDevice);      

NV_STATUS nvGpuOpsFreeDupedHandle(struct gpuAddressSpace *sourceVaspace,
                                  NvHandle hPhysHandle);

NV_STATUS nvGpuOpsFreeOpsHandle(struct gpuSession *session,
                                struct gpuAddressSpace *vaSpace);

NV_STATUS nvGpuOpsGpuMallocWithHandles(struct gpuAddressSpace * vaSpace, NvHandle physHandle, 
                                       NvLength length, NvU64 *gpuOffset, void **cpuPtr);

NV_STATUS nvGpuOpsGetAttachedGpus(NvU8 *guidList, unsigned *numGpus);

NV_STATUS nvGpuOpsGetGpuArch(NvU8 *pUuid, unsigned uuidLength, NvU32 *pGpuArch);

NV_STATUS nvGpuOpsServiceDeviceInterruptsRM(struct gpuChannel *channel);

NV_STATUS nvGpuOpsCheckEccErrorSlowpath(struct gpuChannel * channel, NvBool *bEccDbeSet);

NV_STATUS nvGpuOpsKillChannel(struct gpuChannel * channel);

NV_STATUS nvGpuOpsSetPageDirectory(struct gpuAddressSpace * vaSpace,
                                   NvU64 physAddress, unsigned numEntries,
                                   NvBool bVidMemAperture);

NV_STATUS nvGpuOpsGetGmmuFmt(struct gpuAddressSpace * vaSpace, void ** pFmt);

#endif /* _NV_GPU_OPS_H_*/
